// copyright (c) 2019-2021 hors<horsicq@gmail.com>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
#ifndef PE_SCRIPT_H
#define PE_SCRIPT_H

#include "msdos_script.h"
#include "xpe.h"

class PE_Script : public MSDOS_Script
{
    Q_OBJECT

public:
    explicit PE_Script(XPE *pPE);
    ~PE_Script();

public slots:
    quint16 getNumberOfSections();
    QString getSectionName(quint32 nNumber);
    quint32 getSectionVirtualSize(quint32 nNumber);
    quint32 getSectionVirtualAddress(quint32 nNumber);
    quint32 getSectionFileSize(quint32 nNumber);
    quint32 getSectionFileOffset(quint32 nNumber);
    quint32 getSectionCharacteristics(quint32 nNumber);
    quint32 getNumberOfResources();
    bool isSectionNamePresent(QString sSectionName);
    bool isSectionNamePresentExp(QString sSectionName);
    bool isNET();
    bool isPEPlus();
    bool is64();
    virtual QString getGeneralOptions();
    quint32 getResourceIdByNumber(quint32 nNumber);
    QString getResourceNameByNumber(quint32 nNumber);
    qint64 getResourceOffsetByNumber(quint32 nNumber);
    qint64 getResourceSizeByNumber(quint32 nNumber);
    quint32 getResourceTypeByNumber(quint32 nNumber);
    bool isNETStringPresent(QString sString);
    bool isNETUnicodeStringPresent(QString sString);
    qint32 getNumberOfImports();
    QString getImportLibraryName(quint32 nNumber);
    bool isLibraryPresent(QString sLibraryName);
    bool isLibraryFunctionPresent(QString sLibraryName,QString sFunctionName);
    QString getImportFunctionName(quint32 nImport,quint32 nFunctionNumber);
    qint32 getImportSection();
    qint32 getExportSection();
    qint32 getResourceSection();
    qint32 getEntryPointSection();
    qint32 getRelocsSection();
    quint8 getMajorLinkerVersion();
    quint8 getMinorLinkerVersion();
    QString getManifest();
    QString getVersionStringInfo(QString sKey);
    qint32 getNumberOfImportThunks(quint32 nNumber);
    qint64 getResourceNameOffset(QString sName);
    bool isResourceNamePresent(QString sName);
    bool isResourceGroupNamePresent(QString sName);
    bool isResourceGroupIdPresent(quint32 nID);
    QString getCompilerVersion();
    bool isConsole();
    bool isSignedFile();
    QString getSectionNameCollision(QString sString1,QString sString2);
    qint32 getSectionNumber(QString sSectionName);
    qint32 getSectionNumberExp(QString sSectionName);
    bool isDll();
    bool isDriver();
    QString getNETVersion();
    bool compareEP_NET(QString sSignature,qint64 nOffset=0);
    quint32 getSizeOfCode();
    quint32 getSizeOfUninitializedData();
    QString getPEFileVersion(QString sFileName);
    QString getFileVersion();
    QString getFileVersionMS();
    qint64 calculateSizeOfHeaders();
    bool isExportFunctionPresent(QString sFunctionName);
    bool isExportFunctionPresentExp(QString sFunctionName);

    bool isExportPresent();
    bool isTLSPresent();
    bool isImportPresent();
    bool isResourcesPresent();

private:
    XPE *pPE;
    qint64 g_nBaseAddress;
    qint32 g_nNumberOfSections;
    XPE::CLI_INFO g_cliInfo;
    QList<XPE::RESOURCE_RECORD> listResourceRecords;
    qint32 nNumberOfResources;
    QList<XPE_DEF::IMAGE_SECTION_HEADER> listSectionHeaders;
    QList<XPE::SECTION_RECORD> listSectionRecords;
    QList<QString> listSectionNameStrings;
    QList<XPE::IMPORT_HEADER> listImportHeaders;
    qint32 nNumberOfImports;
    XPE::RESOURCES_VERSION resourcesVersion;
    bool bIsNETPresent;
    bool bIs64;
    bool bIsDll;
    bool bIsDriver;
    bool bIsConsole;
    bool bIsSignPresent;
    bool bIsExportPresent;
    bool bIsTLSPresent;
    bool bIsImportPresent;
    bool bIsResourcesPresent;
    QString sGeneralOptions;
    qint32 nImportSection;
    qint32 nExportSection;
    qint32 nResourcesSection;
    qint32 nEntryPointSection;
    qint32 nRelocsSection;
    quint8 nMajorLinkerVersion;
    quint8 nMinorLinkerVersion;
    quint32 nSizeOfCode;
    quint32 nSizeOfUninitializedData;
    QString sCompilerVersion;
    QString sFileVersion;
    QString sFileVersionMS;
    qint32 nCalculateSizeOfHeaders;
    XPE::EXPORT_HEADER exportHeader;
    QList<QString> listExportFunctionNameStrings;
};

#endif // PE_SCRIPT_H
